#ifndef GRID_H
#define GRID_H


#include <stdbool.h>

#include "dist_array.h"
#include "rectangle.h"
#include "swe.h"
#include "typedefs.h"


struct grid_t {
    int_t
        level,
        n_children,
        total_refinement,
        border,
        regrid_ticks;

    point_t
        origin,
        origin_in_parent,
        global_end;

    length_t
        length,
        length_in_parent;

    real_t
        dy,
        dx,
        dt;

    dist_array_t
        *n,
        *pn,
        *pnu,
        *pnv,
        *pnuv,
        *du,
        *dv,
        *u,
        *v;

    struct grid_t *parent;
    struct grid_t **children;

    // TODO: Borders?
};


typedef enum border_t {
    NORTH,
    SOUTH,
    WEST,
    EAST,
} border_t;


typedef enum wave_t {
    edge,
    constant_length_edge,
    center,
    middle,
} wave_t;


void update_child_grid(grid_t *child, grid_t *parent, point_t origin_in_parent,
                       length_t length_in_parent, int_t refinement);

grid_t * new_child_grid(grid_t *parent, int_t refinement,
                        point_t origin_in_parent, length_t length_in_parent);

grid_t * new_root_grid(point_t origin, length_t length,
                       real_t dy, real_t dx, real_t dt);

void new_grid_common(grid_t *grid);
void free_grid(grid_t *grid);
void add_child_grid(grid_t *parent, point_t origin, length_t length);
// void remove_child_grid(grid_t *child, grid_t *parent);
void downsample(grid_t *parent, grid_t *child);

void initialize(grid_t *grid, length_t length, real_t normal_height,
                real_t wave_height, wave_t wave);

void set_grid_borders(grid_t *child, dist_array_t *a_p, dist_array_t *a_c,
                      int_t t, int_t refinement, int_t buffer);

void set_grid_border(grid_t *child, dist_array_t *a_p, dist_array_t *a_c,
                     int_t t, int_t refinement, border_t border, int_t buffer);

void initialize_child_grids(grid_t *parent, grid_t *child, int_t r);

void initialize_child_grid(grid_t *child, dist_array_t *a_c,
                           dist_array_t *a_p, int_t r);

void write_output(grid_t *grid, const char *filename);

void regrid(grid_t *grid, real_t normal_height, real_t wave_height,
            real_t height_threshold, real_t amount_threshold);

void add_child_grids(grid_t *grid, rectangle_t **accepted, int_t n_accepted);


#endif
