#ifndef RECTANGLE_H
#define RECTANGLE_H


#include "typedefs.h"
#include "grid.h"


typedef struct signature_t {
    int_t
        *y,
        *x;
} signature_t;


struct rectangle_t {
    point_t origin;
    length_t length;

    signature_t
        signature,
        laplacian;
};


typedef enum direction_t {
    horizontal,
    vertical,
} direction_t;


bool is_zero_crossing(int_t a, int_t b);
rectangle_t * new_rectangle(point_t origin, length_t length);

void evaluate_rectangle(rectangle_t *r, grid_t *grid, bool **flagged,
                        real_t threshold, rectangle_t **accepted,
                        int_t *n_accepted);

void free_rectangle(rectangle_t *r);
void get_laplacian(int_t length, int_t *signature, int_t *laplacian);

void get_max_zero_crossing(int_t *max_zero_crossing_value, int_t *split_index,
                           direction_t *split_direction, int_t length,
                           int_t *laplacian, direction_t new_direction,
                           int_t origin);

bool find_hole(rectangle_t *r, int_t *split_index,
               direction_t *split_direction);

bool find_inflection_point(rectangle_t *r, int_t *split_index,
                           direction_t *split_direction);

bool find_hole_in_direction(int_t *signature, int_t length, int_t *split_index,
                            direction_t *split_direction,
                            direction_t new_direction, int_t origin);

void accept_rectangle(rectangle_t *r, rectangle_t **accepted,
                      int_t *n_accepted);

int compare_rectangles(const void *p1, const void *p2);


#endif
